// main board view - handles piece drawing and display, general gameplay stuff

#import "TrisBoardView.h"

@interface TrisApp : UIApplication {}
- (void)setNextPiece:(int)p;
- (void)cleared;
- (void)dropped:(int)dist;
- (void)lost;
@end

@implementation TrisBoardView

CGPoint dragStartPoint;
int dragStartRot;
int lastDragMovY;
int lastDragMovX;

double lastUpTime;
double lastDownTime;

NSTimer *gameTim;

int cPcXs[4];
int cPcYs[4];

int nextPiece;

double lastTicked;

float dropInterval;

BOOL running;

int nextPieces[7];
int cNextPiece;

- (id)initWithFrame:(CGRect)frame app:(TrisApp *)a
{
	if(self == [super initWithFrame:frame])
	{
		app = a;
		
		bg = [[UIImage applicationImageNamed:@"board.png"] retain];
		
		running = NO;
		
		dropInterval = 0.5;
		
		cNextPiece = 7;
		
		pieces = [[NSArray arrayWithObjects:
			[UIImage applicationImageNamed:@"blockO.png"],
			[UIImage applicationImageNamed:@"blockL.png"],
			[UIImage applicationImageNamed:@"blockJ.png"],
			[UIImage applicationImageNamed:@"blockT.png"],
			[UIImage applicationImageNamed:@"blockS.png"],
			[UIImage applicationImageNamed:@"blockZ.png"],
			[UIImage applicationImageNamed:@"blockI.png"],
			[UIImage applicationImageNamed:@"glowO.png"],
			[UIImage applicationImageNamed:@"glowL.png"],
			[UIImage applicationImageNamed:@"glowJ.png"],
			[UIImage applicationImageNamed:@"glowT.png"],
			[UIImage applicationImageNamed:@"glowS.png"],
			[UIImage applicationImageNamed:@"glowZ.png"],
			[UIImage applicationImageNamed:@"glowI.png"],
			nil] retain];
		
		// blurred-out background (for menu screen)
		blurBgVu = [[[UIImageView alloc] initWithImage:[UIImage applicationImageNamed:@"board-blur.png"]] retain];
		[blurBgVu setFrame:CGRectMake(0,0,230,460)];
		[blurBgVu setAlpha:1.0];
		[[blurBgVu _layer] setZPosition:2.0];
		[self addSubview:blurBgVu];
		
		int p;
		for(p = 0;p<[pieces count];p++)
		{
			[[pieces objectAtIndex:p] retain];
		}
		
		srand(time(NULL));
		
		nextPiece = (rand() % 7) + 1;
		cPiece = [[TrisPiece pieceWithKind:nextPiece imageSource:pieces] retain];
		[self chooseNextPiece];
		
		[cPiece setPosNoAnim:CGPointMake(115.0,-46.0)];
		[[self _layer] addSublayer:[cPiece layer]];
		
		// set up the array of cells
		cells = [[NSMutableArray arrayWithCapacity:10] retain];
		int i,j;
		for(i=0;i<10;i++)
		{
			NSMutableArray *cellCol = [[NSMutableArray arrayWithCapacity:20] retain];
			for(j=0;j<20;j++)
			{
				TrisCell *c = [[[TrisCell alloc] init] retain];
				[cellCol addObject:c];
			}
			[cells addObject:cellCol];
		}
		[self startHeartbeat:@selector(tick) inRunLoopMode:(id)kCFRunLoopDefaultMode];
		
		[self setNeedsDisplay];
	}
	
	return self;
}

- (BOOL)value:(int)v notUsedIn:(int *)a
{
	int i;
	for(i = 0;i < 7;i++)
	{
		if(a[i] == v)
			return NO;
	}
	return YES;
}

- (void)chooseNextPiece
{
	// this algorithm is really ugly
	if(cNextPiece == 7)
	{
		int i;
		for(i=0;i<7;i++)
			nextPieces[i] = 0;
		for(i=0;i<7;i++)
		{
			int v = (rand() % 7) + 1;
			BOOL nu = NO;
			int j;
			while(!nu)
			{
				nu = YES;
				v = (rand() % 7) + 1;
				for(j=0;j<7;j++)
					if(nextPieces[j] == v)
						nu = NO;
			}
			nextPieces[i] = v;
		}
		cNextPiece = 0; // ...since it gets added to next tick
	}
	nextPiece = nextPieces[cNextPiece];
	cNextPiece++;
	
	[app setNextPiece:nextPiece];
}

- (void)resetPiece
{
	if(clearingPiece != nil)
		[self rmClearing];
	clearingPiece = cPiece;
	
	// slowly fade out the last landed piece
	[LKTransaction begin];
	[LKTransaction setValue:[NSNumber numberWithFloat:0.7] forKey:@"animationDuration"];
	[[clearingPiece layer] setValue:[NSNumber numberWithFloat:0.0] forKey:@"opacity"];
	[LKTransaction commit];
	
	// ... and create a new one to fall from the top
	cPiece = [[TrisPiece pieceWithKind:nextPiece imageSource:pieces] retain];
	[cPiece setPosNoAnim:CGPointMake(115.0,-46.0)];
	[[self _layer] addSublayer:[cPiece layer]];
	[cPiece setRot:0];
	[self chooseNextPiece];
	
	// check for completed rows
	while([self checkRows]){}
	
	[self setNeedsDisplay];
}

- (void)rmClearing
{
	if(clearingPiece == nil)
		return;
	[[clearingPiece layer] removeFromSuperlayer];
	[clearingPiece release];
}

- (void)doDrop:(id)u
{
	if([self pieceLandedAtX:[cPiece xCell] Y:[cPiece yCell]])
	{
		// move it from the visible layer thing to the actual cell array
		[self placePiece];
		
		// and respawn it
		[self resetPiece];
		
	}
	else
		[cPiece drop];
}

- (BOOL)checkRows
{
	NSMutableArray *cols[10];
	int x,y;
	for(x=0;x<10;x++)
		cols[x] = (NSMutableArray *)[cells objectAtIndex:x];
	for(y=19;y>=0;y--)
	{
		BOOL row = YES;
		for(x=0;x<10;x++)
		{
			if([(TrisCell *)[cols[x] objectAtIndex:y] getContents] == 0)
				row = NO;
		}
		if(row)
		{
			// clear that row, insert a new one at the top
			for(x=0;x<10;x++)
			{
				TrisCell *c = [cols[x] objectAtIndex:y];
				[c release];
				[cols[x] removeObjectAtIndex:y];
				[cols[x] insertObject:[[[TrisCell alloc] init] retain] atIndex:0];
			}
			y+=1;
			[app cleared];
			return YES;
		}
	}
	return NO;
}

#if defined ASPEN && !defined USE_LEGACY_EVENT
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	UITouch *touch = [[event allTouches] anyObject];
	dragStartPoint = [touch locationInView:self];
	dragStartRot = [cPiece rot];
	lastDragMovX = (int)dragStartPoint.x;
	lastDragMovY = (int)dragStartPoint.y;
	lastDownTime = CFAbsoluteTimeGetCurrent();
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{  
	double t = CFAbsoluteTimeGetCurrent();
	
	if(t > lastDownTime + 0.3)
		lastUpTime = 0;
	else
	{
		UITouch *touch = [[event allTouches] anyObject];
		CGPoint p = [touch locationInView:self];
		if(abs(p.x - dragStartPoint.x) < 10 && abs(p.y - dragStartPoint.y) < 10)
			[self rotatePieceRight:YES];
	}
}


- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{ 	
	UITouch *touch = [[event allTouches] anyObject];
	CGPoint p = [touch locationInView:self];
 	float dx = p.x - dragStartPoint.x;
	float dy = p.y - dragStartPoint.y;
	/*if(abs(p.y - lastDragRotY) > 40)
	{
		
		// moved all this to rotatePieceRight:
	 
		lastDragRotY = p.y;
		lastUpTime = 0;
	}*/
	int yDir = -1;
	if(p.y > lastDragMovY)
		yDir = 1;
	if(abs(p.y - lastDragMovY) > 12 && [self pieceCanMoveYDir:yDir])
	{
		[cPiece movY:yDir];
		lastDragMovY = p.y;
		lastUpTime = 0;
	}
	int xDir = -1;
	if(p.x > lastDragMovX)
		xDir = 1;
	if(abs(p.x - lastDragMovX) > 25 && [self pieceCanMoveXDir:xDir])
	{
		[cPiece movX:xDir];
		lastDragMovX = p.x;
		lastUpTime = 0;
	}
	int x = p.x;
	x = x - (x%23);

}
#else

- (void)mouseDown:(GSEvent *)evt
{
	dragStartPoint = GSEventGetLocationInWindow(evt);
	dragStartRot = [cPiece rot];
	lastDragMovX = (int)dragStartPoint.x;
	lastDragMovY = (int)dragStartPoint.y;
	lastDownTime = CFAbsoluteTimeGetCurrent();
}

- (void)mouseUp:(GSEvent *)evt
{
	double t = CFAbsoluteTimeGetCurrent();
	
	if(t > lastDownTime + 0.3)
		lastUpTime = 0;
	else
	{
		CGPoint p = GSEventGetLocationInWindow(evt);
		if(abs(p.x - dragStartPoint.x) < 10 && abs(p.y - dragStartPoint.y) < 10)
			[self rotatePieceRight:YES];
	}
}

- (void)mouseDragged:(GSEvent *)evt
{
	CGPoint p = GSEventGetLocationInWindow(evt);
	float dx = p.x - dragStartPoint.x;
	float dy = p.y - dragStartPoint.y;
	/*if(abs(p.y - lastDragRotY) > 40)
	{
		
		// moved all this to rotatePieceRight:
	 
		lastDragRotY = p.y;
		lastUpTime = 0;
	}*/
	int yDir = -1;
	if(p.y > lastDragMovY)
		yDir = 1;
	if(abs(p.y - lastDragMovY) > 12 && [self pieceCanMoveYDir:yDir])
	{
		[cPiece movY:yDir];
		lastDragMovY = p.y;
		lastUpTime = 0;
	}
	int xDir = -1;
	if(p.x > lastDragMovX)
		xDir = 1;
	if(abs(p.x - lastDragMovX) > 25 && [self pieceCanMoveXDir:xDir])
	{
		[cPiece movX:xDir];
		lastDragMovX = p.x;
		lastUpTime = 0;
	}
	int x = p.x;
	x = x - (x%23);
}
#endif


- (void)rotatePieceRight:(BOOL)r
{
	if(r)
		[cPiece rotR];
	else
		[cPiece rotL];
	[self loadPieceCoords];
	int cX = [cPiece xCell];
	int cY = [cPiece yCell];
	int i;
	BOOL needRotReset = NO;
	// make sure the piece can't be rotated through a wall / another piece
	for(i=0;i<4;i++)
	{
		int cPx = cX + cPcXs[i];
		int cPy = cY + cPcYs[i];
		if(cPx < 0)
		{
			if([self pieceCanMoveXDir:1])
			{
				[cPiece movX:1];
				[self loadPieceCoords];
				if([cPiece xCell] + cPcXs[i] < 0)
					if([self pieceCanMoveXDir:1])
						[cPiece movX:1];
					else
						needRotReset = YES;
			}
			else
				needRotReset = YES;
			break;
		}
		else if(cPx > 9)
		{
			if([self pieceCanMoveXDir:-1])
			{
				[cPiece movX:-1];
				int j;
				for(j=0;j<4;j++)
				{
					if([cPiece xCell] + cPcXs[j] > 9)
						if([self pieceCanMoveXDir:-1])
							[cPiece movX:-1];
						else
							needRotReset = YES;
				}
			}
			else
				needRotReset = YES;
			break;
		}
		else if(cPy > 19)
		{
			needRotReset = YES;
		}
		else if((cPx >= 0 && cPy >= 0 && cPx <= 9 && cPy <= 19) && [(TrisCell *)[(NSArray *)[cells objectAtIndex:cPx] objectAtIndex:cPy] getContents] != 0)
		{
			needRotReset = YES;
		}
	}
	if(needRotReset)
		if(r)
			[cPiece rotL];
		else
			[cPiece rotR];
}


- (void)reset
{
	int x,y;
	for(x=0;x<10;x++)
	{
		NSMutableArray *cCol = [cells objectAtIndex:x];
		for(y=0;y<20;y++)
		{
			[(TrisCell *)[cCol objectAtIndex:y] setContents:0];
		}
	}
	[self resetPiece];
}

- (void)pause
{
	running = NO;
	UIAlphaAnimation *aa = [[UIAlphaAnimation alloc] initWithTarget:blurBgVu];
	[aa setStartAlpha:0.0];
	[aa setEndAlpha:1.0];
	[[UIAnimator sharedAnimator] addAnimation:aa withDuration:0.4 start:YES];
	[aa release];
}

- (void)slowBlur
{
	UIAlphaAnimation *aa = [[UIAlphaAnimation alloc] initWithTarget:blurBgVu];
	[aa setStartAlpha:0.0];
	[aa setEndAlpha:1.0];
	[[UIAnimator sharedAnimator] addAnimation:aa withDuration:1.5 start:YES];
	[aa release];
}

- (void)unpause
{
	UIAlphaAnimation *aa = [[UIAlphaAnimation alloc] initWithTarget:blurBgVu];
	[aa setStartAlpha:1.0];
	[aa setEndAlpha:0.0];
	[[UIAnimator sharedAnimator] addAnimation:aa withDuration:0.3 start:YES];
	[NSTimer scheduledTimerWithTimeInterval:0.4 target:self selector:@selector(resume:) userInfo:nil repeats:NO];
}

- (void)setDropInterval:(float)i
{
	dropInterval = i;
}

- (void)resume:(id)uu
{
	running = YES;
}

- (BOOL)pieceCanMoveXDir:(int)d
{
	[self loadPieceCoords];
	int cX = [cPiece xCell];
	int cY = [cPiece yCell];
	int i;
	// check obstruction of each block
	for(i=0;i<4;i++)
	{
		int cbX = cX + cPcXs[i];
		int cbY = cY + cPcYs[i];
		if((d == -1 && cbX <= 0) || (d == 1 && cbX >= 9))
			return NO;
		if(cbY >= 0 && cbY <= 19 && cbX + d >= 0 && cbX + d <= 9)
			if([(TrisCell *)[(NSArray *)[cells objectAtIndex:(cbX + d)] objectAtIndex:cbY] getContents] > 0)
				return NO;
	}
	return YES;
}

- (BOOL)pieceCanMoveYDir:(int)d
{
	[self loadPieceCoords];
	int cX = [cPiece xCell];
	int cY = [cPiece yCell];
	int i;
	for(i=0;i<4;i++)
	{
		int cbX = cX + cPcXs[i];
		int cbY = cY + cPcYs[i];
		if((d == -1 && cbY < 0) || (d == 1 && cbY >= 19))
			return NO;
		if(cbY + d >= 0 && cbY + d <= 19 && cbX >= 0 && cbX <= 9)
			if([(TrisCell *)[(NSArray *)[cells objectAtIndex:cbX] objectAtIndex:(cbY + d)] getContents] != 0)
				return NO;
	}
	return YES;
}

// fill the offset coordinate arrays
- (void)loadPieceCoords
{
	int ki = [cPiece kind];
	int r = [cPiece rot];
	switch(ki)
	{
		case TRIS_O:
			cPcXs[0] = -1;
			cPcYs[0] = -1;
			cPcXs[1] = -1;
			cPcYs[1] = 0;
			cPcXs[2] = 0;
			cPcYs[2] = -1;
			cPcXs[3] = 0;
			cPcYs[3] = 0;
			break;
		case TRIS_L:
			switch(r)
		{
			case 0: // n
				cPcXs[0] = -1;
				cPcYs[0] = -2;
				cPcXs[1] = -1;
				cPcYs[1] = -1;
				cPcXs[2] = -1;
				cPcYs[2] = 0;
				cPcXs[3] = 0;
				cPcYs[3] = 0;
				break;
			case 1: // e
				cPcXs[0] = -1;
				cPcYs[0] = -1;
				cPcXs[1] = 0;
				cPcYs[1] = -1;
				cPcXs[2] = 1;
				cPcYs[2] = -1;
				cPcXs[3] = -1;
				cPcYs[3] = 0;
				break;
			case 2: // s
				cPcXs[0] = 0;
				cPcYs[0] = 1;
				cPcXs[1] = 0;
				cPcYs[1] = 0;
				cPcXs[2] = 0;
				cPcYs[2] = -1;
				cPcXs[3] = -1;
				cPcYs[3] = -1;
				break;
			case 3: // w
				cPcXs[0] = -2;
				cPcYs[0] = 0;
				cPcXs[1] = -1;
				cPcYs[1] = 0;
				cPcXs[2] = 0;
				cPcYs[2] = 0;
				cPcXs[3] = 0;
				cPcYs[3] = -1;
				break;
		}
			break;
		case TRIS_J:
			switch(r)
		{
			case 0: // n
				cPcXs[0] = 0;
				cPcYs[0] = -2;
				cPcXs[1] = 0;
				cPcYs[1] = -1;
				cPcXs[2] = 0;
				cPcYs[2] = 0;
				cPcXs[3] = -1;
				cPcYs[3] = 0;
				break;
			case 1: // e
				cPcXs[0] = -1;
				cPcYs[0] = -1;
				cPcXs[1] = 0;
				cPcYs[1] = 0;
				cPcXs[2] = 1;
				cPcYs[2] = 0;
				cPcXs[3] = -1;
				cPcYs[3] = 0;
				break;
			case 2: // s
				cPcXs[0] = -1;
				cPcYs[0] = -1;
				cPcXs[1] = 0;
				cPcYs[1] = -1;
				cPcXs[2] = -1;
				cPcYs[2] = 0;
				cPcXs[3] = -1;
				cPcYs[3] = 1;
				break;
			case 3: // w
				cPcXs[0] = -2;
				cPcYs[0] = -1;
				cPcXs[1] = -1;
				cPcYs[1] = -1;
				cPcXs[2] = 0;
				cPcYs[2] = 0;
				cPcXs[3] = 0;
				cPcYs[3] = -1;
				break;
		}
			break;
		case TRIS_T:
			switch(r)
		{
			case 0: // n
				cPcXs[0] = -1;
				cPcYs[0] = -1;
				cPcXs[1] = 0;
				cPcYs[1] = -1;
				cPcXs[2] = 1;
				cPcYs[2] = -1;
				cPcXs[3] = 0;
				cPcYs[3] = 0;
				break;
			case 1: // e
				cPcXs[0] = 0;
				cPcYs[0] = -1;
				cPcXs[1] = 0;
				cPcYs[1] = 0;
				cPcXs[2] = 0;
				cPcYs[2] = 1;
				cPcXs[3] = -1;
				cPcYs[3] = 0;
				break;
			case 2: // s
				cPcXs[0] = -2;
				cPcYs[0] = 0;
				cPcXs[1] = -1;
				cPcYs[1] = -1;
				cPcXs[2] = 0;
				cPcYs[2] = 0;
				cPcXs[3] = -1;
				cPcYs[3] = 0;
				break;
			case 3: // w
				cPcXs[0] = -1;
				cPcYs[0] = -2;
				cPcXs[1] = -1;
				cPcYs[1] = -1;
				cPcXs[2] = -1;
				cPcYs[2] = 0;
				cPcXs[3] = 0;
				cPcYs[3] = -1;
				break;
		}
			break;
		case TRIS_S:
			switch(r)
		{
			case 0: // n
				cPcXs[0] = -1;
				cPcYs[0] = -1;
				cPcXs[1] = -1;
				cPcYs[1] = 0;
				cPcXs[2] = 0;
				cPcYs[2] = 0;
				cPcXs[3] = 0;
				cPcYs[3] = 1;
				break;
			case 1: // e
				cPcXs[0] = 0;
				cPcYs[0] = -1;
				cPcXs[1] = -1;
				cPcYs[1] = -1;
				cPcXs[2] = -1;
				cPcYs[2] = 0;
				cPcXs[3] = -2;
				cPcYs[3] = 0;
				break;
			case 2: // s
				cPcXs[0] = -1;
				cPcYs[0] = -2;
				cPcXs[1] = -1;
				cPcYs[1] = -1;
				cPcXs[2] = 0;
				cPcYs[2] = -1;
				cPcXs[3] = 0;
				cPcYs[3] = 0;
				break;
			case 3: // w
				cPcXs[0] = 1;
				cPcYs[0] = -1;
				cPcXs[1] = 0;
				cPcYs[1] = -1;
				cPcXs[2] = 0;
				cPcYs[2] = 0;
				cPcXs[3] = -1;
				cPcYs[3] = 0;
				break;
		}
			break;
		case TRIS_Z:
			switch(r)
		{
			case 0: // n
				cPcXs[0] = -1;
				cPcYs[0] = 0;
				cPcXs[1] = -1;
				cPcYs[1] = 1;
				cPcXs[2] = 0;
				cPcYs[2] = -1;
				cPcXs[3] = 0;
				cPcYs[3] = 0;
				break;
			case 1: // e
				cPcXs[0] = -1;
				cPcYs[0] = -1;
				cPcXs[1] = -2;
				cPcYs[1] = -1;
				cPcXs[2] = 0;
				cPcYs[2] = 0;
				cPcXs[3] = -1;
				cPcYs[3] = 0;
				break;
			case 2: // s
				cPcXs[0] = -1;
				cPcYs[0] = 0;
				cPcXs[1] = -1;
				cPcYs[1] = -1;
				cPcXs[2] = 0;
				cPcYs[2] = -1;
				cPcXs[3] = 0;
				cPcYs[3] = -2;
				break;
			case 3: // w
				cPcXs[0] = 0;
				cPcYs[0] = -1;
				cPcXs[1] = -1;
				cPcYs[1] = -1;
				cPcXs[2] = 1;
				cPcYs[2] = 0;
				cPcXs[3] = 0;
				cPcYs[3] = 0;
				break;
		}
			break;
		case TRIS_I:
			switch(r)
		{
			case 0: // n
				cPcXs[0] = cPcXs[1] = cPcXs[2] = cPcXs[3] = -1;
				cPcYs[0] = -2;
				cPcYs[1] = -1;
				cPcYs[2] = 0;
				cPcYs[3] = 1;
				break;
			case 1: // e
				cPcXs[0] = -2;
				cPcXs[1] = -1;
				cPcXs[2] = 0;
				cPcXs[3] = 1;
				cPcYs[0] = cPcYs[1] = cPcYs[2] = cPcYs[3] = 0;
				break;
			case 2: // s
				cPcXs[0] = cPcXs[1] = cPcXs[2] = cPcXs[3] = 0;
				cPcYs[0] = -2;
				cPcYs[1] = -1;
				cPcYs[2] = 0;
				cPcYs[3] = 1;
				break;
			case 3: // w
				cPcXs[0] = -2;
				cPcXs[1] = -1;
				cPcXs[2] = 0;
				cPcXs[3] = 1;
				cPcYs[0] = cPcYs[1] = cPcYs[2] = cPcYs[3] = 0;
				break;
		}
			break;
	}
}

- (BOOL)pieceLandedAtX:(int)cX Y:(int)cY
{
	[self loadPieceCoords];
	
	int i;
	BOOL l = NO;
	for(i=0;i<4;i++)
	{
		int cbX = cX + cPcXs[i];
		int cbY = cY + cPcYs[i];
		if(cbY >= 19) // hit bottom?
			l = YES;
		if(cbY >= 0 && cbY <= 18 && cbX >= 0 && cbX <= 9) // don't try to get the cell if the piece is out-of-bounds (though it shouldn't be anyway)
		{
			TrisCell *c = [[cells objectAtIndex:cbX] objectAtIndex:(cbY + 1)];
			if([c getContents] > 0)
				l = YES;
		}
	}
	return l;
}

- (void)placePiece
{
	int cX = [cPiece xCell];
	int cY = [cPiece yCell];
	int i;
	for(i=0;i<4;i++)
	{
		if(cY + cPcYs[i] >= 0)
			[(TrisCell *)[[cells objectAtIndex:(cX + cPcXs[i])] objectAtIndex:(cY + cPcYs[i])] setContents:[cPiece kind]];
		else
		{
			[app lost];
			running = NO;
			return;
		}
	}
	[app dropped:[cPiece getYDragDistance]];
	[self setNeedsDisplay];
}

- (void)drawRect:(CGRect)r
{
	[bg draw1PartImageInRect:CGRectMake(0,0,230,480)];
	
	// draw the static cells... thankfully, the LKLayers take care of themselves
	int x,y;
	for(x=0;x<10;x++)
	{
		NSArray *cCol = [cells objectAtIndex:x];
		for(y=0;y<20;y++)
		{
			int v = [(TrisCell *)[cCol objectAtIndex:y] getContents];
			if(v != 0)
				[[pieces objectAtIndex:v-1] draw1PartImageInRect:CGRectMake(x*23,y*23,23,23)];
		}
	}
}

- (void)dealloc
{
	int i,j;
	for(i=0;i<10;i++)
	{
		NSMutableArray *cCol = [cells objectAtIndex:i];
		for(j=0;j<20;j++)
			[[cCol objectAtIndex:j] release];
		[cCol release];
	}
	
	int p;
	for(p = 0;p<[pieces count];p++)
	{
		[[pieces objectAtIndex:p] release];
	}
	
	[cells release];
	[cPiece release];
	[pieces release];
	[blurBgVu release];
	[bg release];
	[super dealloc];
}

- (void)tick
{
	if(running)
	{
		double t = CFAbsoluteTimeGetCurrent();
		if(t > lastTicked + (double)dropInterval)
		{
			[self doDrop:nil];
			lastTicked = t;
		}
	}
}

@end