// piece class - manages the sublayers to display the currently-falling piece

#import "TrisPiece.h"

@implementation TrisPiece

+ (TrisPiece *)pieceWithKind:(int)ki imageSource:(NSArray *)sou
{
	return [[[TrisPiece alloc] initWithKind:ki imageSource:sou] autorelease];
}

- (id)initWithKind:(int)ki imageSource:(NSArray *)sou
{
	if(self = [super init])
	{
		iSrc = sou;
		knd = ki;
		cRot = 0;
		minY = -2;
		mainLyr = [[LKLayer layer] retain];
		[mainLyr setFrame:CGRectMake(0.0,0.0,23.0,23.0)]; // since it's not masking to bounds, the frame shouldn't matter, but hey
		[mainLyr setMasksToBounds:NO];
		// the image source stores the various-styled blocks and their corresponding glows
		id blockRef = (id)[(UIImage *)[sou objectAtIndex:ki-1] imageRef];
		id glowRef = (id)[(UIImage *)[sou objectAtIndex:ki+6] imageRef];
		int i;
		for(i=0;i<4;i++)
		{
			blocks[i] = [[LKLayer layer] retain];
			[blocks[i] setContents:blockRef];
			[mainLyr addSublayer:blocks[i]];
			glows[i] = [[LKLayer layer] retain];
			[glows[i] setContents:glowRef];
			[mainLyr addSublayer:glows[i]];
		}
		[self setKind:ki];
	}
	return self;
}

- (int)xCell
{
	return tX;
}

- (int)yCell
{
	return tY;
}

- (void)drop
{
	CGRect fr = [mainLyr frame];
	tY += 1;
	minY += 1;
	[self setPos:CGPointMake(fr.origin.x,fr.origin.y+23)];
}

- (LKLayer *)layer
{
	return mainLyr;
}

- (int)rot
{
	return cRot;
}

- (int)kind
{
	return knd;
}

- (int)getYDragDistance
{
	return tY - minY;
}

- (void)setKind:(int)ki
{
	knd = ki;
	id blockRef = (id)[(UIImage *)[iSrc objectAtIndex:ki-1] imageRef];
	id glowRef = (id)[(UIImage *)[iSrc objectAtIndex:ki+6] imageRef];
	int i;
	for(i=0;i<4;i++)
	{
		[blocks[i] setContents:blockRef];
		[glows[i] setContents:glowRef];
	}
	[self setRot:cRot];
}

- (void)rotL
{
	cRot -= 1;
	if(cRot < 0)
		cRot = 3;
	[self setRot:cRot];
}

- (void)rotR
{
	cRot += 1;
	if(cRot > 3)
		cRot = 0;
	[self setRot:cRot];
}

- (void)movX:(int)dir
{
	CGRect r = [mainLyr frame];
	tX += dir;
	[self setPos:CGPointMake(r.origin.x+(23*dir),r.origin.y)];
}

- (void)movY:(int)dir
{
	if(tY + dir >= minY)
	{
		CGRect r = [mainLyr frame];
		tY += dir;
		[self setPos:CGPointMake(r.origin.x,r.origin.y+(23*dir))];
	}
}

- (void)setRot:(int)r
{
	cRot = r;
	CGAffineTransform idt = CGAffineTransformIdentity;
	CGAffineTransform rot = CGAffineTransformMakeRotation(1.570796326795); // 90-degree rotation
	[LKTransaction begin];
	[LKTransaction setValue:(id)kCFBooleanTrue forKey:@"disableActions"]; // no animation of block movements during rotation - it looks weird
	int i;
	for(i=0;i<4;i++)
		[glows[i] setAffineTransform:idt];
	switch(knd)
	{
		case TRIS_O:
			[blocks[0] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
			[blocks[1] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
			[blocks[2] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
			[blocks[3] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
			[glows[0] setFrame:CGRectMake(-27.0,-8.0,31.0,16.0)];
			[glows[1] setFrame:CGRectMake(-4.0,-8.0,31.0,16.0)];
			[glows[2] setFrame:CGRectMake(-15.6,-19.5,31.0,16.0)];
			[glows[2] setAffineTransform:rot];
			[glows[3] setHidden:NO];
			[glows[3] setFrame:CGRectMake(-15.6,3.0,31.0,16.0)];
			[glows[3] setAffineTransform:rot];
			break;
		case TRIS_L:
			switch(r)
			{
				case 0: // n
					[blocks[0] setFrame:CGRectMake(-23.0,-46.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-27.0,-31.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(-27.0,-8.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-15.6,3.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
				case 1: // e
					[blocks[0] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(23.0,-23.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-27.0,-8.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(7.4,-19.5,31.0,16.0)];
					[glows[1] setAffineTransform:rot];
					[glows[2] setFrame:CGRectMake(-15.6,-19.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
				case 2: // s
					[blocks[0] setFrame:CGRectMake(0.0,23.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-4.0,-8.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(-4.0,15.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-15.6,-19.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
				case 3: // w
					[blocks[0] setFrame:CGRectMake(-46.0,0.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-4.0,-8.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(-15.6,3.5,31.0,16.0)];
					[glows[1] setAffineTransform:rot];
					[glows[2] setFrame:CGRectMake(-38.6,3.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					[glows[3] setHidden:YES];
					break;
			}
			[glows[3] setHidden:YES];
			break;
		case TRIS_J:
			switch(r)
			{
				case 0: // n
					[blocks[0] setFrame:CGRectMake(0.0,-46.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-4.0,-31.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(-4.0,-8.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-15.6,3.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
				case 1: // e
					[blocks[0] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(23.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-27.0,-8.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(7.4,3.5,31.0,16.0)];
					[glows[1] setAffineTransform:rot];
					[glows[2] setFrame:CGRectMake(-15.6,3.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
				case 2: // s
					[blocks[0] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(-23.0,23.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-27.0,-8.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(-27.0,15.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-15.6,-19.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
				case 3:
					[blocks[0] setFrame:CGRectMake(-46.0,-23.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-4.0,-8.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(-38.6,-19.5,31.0,16.0)];
					[glows[1] setAffineTransform:rot];
					[glows[2] setFrame:CGRectMake(-15.6,-19.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
			}
			[glows[3] setHidden:YES];
			break;
		case TRIS_T:
			switch(r)
			{
				case 0: // n
					[blocks[0] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(23.0,-23.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-15.6,-19.5,31.0,16.0)];
					[glows[0] setAffineTransform:rot];
					[glows[1] setFrame:CGRectMake(7.4,-19.5,31.0,16.0)];
					[glows[1] setAffineTransform:rot];
					[glows[2] setFrame:CGRectMake(-4.0,-8.0,31.0,16.0)];
					break;
				case 1: // e
					[blocks[0] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,23.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-15.6,3.5,31.0,16.0)];
					[glows[0] setAffineTransform:rot];
					[glows[1] setFrame:CGRectMake(-4.0,15.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-4.0,-8.0,31.0,16.0)];
					break;
				case 2: // s
					[blocks[0] setFrame:CGRectMake(-46.0,0.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-38.6,3.5,31.0,16.0)];
					[glows[0] setAffineTransform:rot];
					[glows[1] setFrame:CGRectMake(-15.6,3.5,31.0,16.0)];
					[glows[1] setAffineTransform:rot];
					[glows[2] setFrame:CGRectMake(-27,-8.0,31.0,16.0)];
					break;
				case 3: // w
					[blocks[0] setFrame:CGRectMake(-23.0,-46.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-15.6,-19.5,31.0,16.0)];
					[glows[0] setAffineTransform:rot];
					[glows[1] setFrame:CGRectMake(-27.0,-8.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-27.0,-31.0,31.0,16.0)];
					break;
			}
			[glows[3] setHidden:YES];
			break;
		case TRIS_S:
			switch(r)
			{
				case 0: // n
					[blocks[0] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(0.0,23.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-27.0,-8.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(-4.0,15.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-15.6,3.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
				case 1: // e
					[blocks[0] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(-46.0,0.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-15.6,-19.5,31.0,16.0)];
					[glows[0] setAffineTransform:rot];
					[glows[1] setFrame:CGRectMake(-27.0,-8.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-38.6,3.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
				case 2: // s
					[blocks[0] setFrame:CGRectMake(-23.0,-46.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-27.0,-31.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(-4.0,-8.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-15.6,-19.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
				case 3: // w
					[blocks[0] setFrame:CGRectMake(23.0,-23.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(7.4,-19.5,31.0,16.0)];
					[glows[0] setAffineTransform:rot];
					[glows[1] setFrame:CGRectMake(-4.0,-8.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-15.6,3.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
			}
			[glows[3] setHidden:YES];
			break;
		case TRIS_Z:
			switch(r)
			{
				case 0: // n
					[blocks[0] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-4.0,-8.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(-27.0,15.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-15.6,3.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
				case 1: // e
					[blocks[0] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-46.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-38.6,-19.5,31.0,16.0)];
					[glows[0] setAffineTransform:rot];
					[glows[1] setFrame:CGRectMake(-27.0,-8.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-15.6,3.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
				case 2: // s
					[blocks[0] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(0.0,-46.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-4.0,-31.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(-27.0,-8.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-15.6,-19.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
				case 3: // w
					[blocks[0] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(23.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-15.6,-19.5,31.0,16.0)];
					[glows[0] setAffineTransform:rot];
					[glows[1] setFrame:CGRectMake(-4.0,-8.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(7.4,3.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
			}
			[glows[3] setHidden:YES];
			break;
		case TRIS_I:
			switch(r)
			{
				case 0: // n
					[blocks[0] setFrame:CGRectMake(-23.0,-46.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(-23.0,23.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-27.0,-31.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(-27.0,-8.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-27.0,15.0,31.0,16.0)];
					break;
				case 1:
					[blocks[0] setFrame:CGRectMake(-46.0,-23.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(23.0,-23.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-38.6,-19.5,31.0,16.0)];
					[glows[0] setAffineTransform:rot];
					[glows[1] setFrame:CGRectMake(7.4,-19.5,31.0,16.0)];
					[glows[1] setAffineTransform:rot];
					[glows[2] setFrame:CGRectMake(-15.6,-19.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
				case 2:
					[blocks[0] setFrame:CGRectMake(0.0,-46.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(0.0,-23.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(0.0,23.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-4.0,-31.0,31.0,16.0)];
					[glows[1] setFrame:CGRectMake(-4.0,-8.0,31.0,16.0)];
					[glows[2] setFrame:CGRectMake(-4.0,15.0,31.0,16.0)];
					break;
				case 3:
					[blocks[0] setFrame:CGRectMake(-46.0,0.0,23.0,23.0)];
					[blocks[1] setFrame:CGRectMake(-23.0,0.0,23.0,23.0)];
					[blocks[2] setFrame:CGRectMake(0.0,0.0,23.0,23.0)];
					[blocks[3] setFrame:CGRectMake(23.0,0.0,23.0,23.0)];
					[glows[0] setFrame:CGRectMake(-38.6,3.5,31.0,16.0)];
					[glows[0] setAffineTransform:rot];
					[glows[1] setFrame:CGRectMake(7.4,3.5,31.0,16.0)];
					[glows[1] setAffineTransform:rot];
					[glows[2] setFrame:CGRectMake(-15.6,3.5,31.0,16.0)];
					[glows[2] setAffineTransform:rot];
					break;
			}
			[glows[3] setHidden:YES];
			break;
		default:
			
			break;
	}
	[LKTransaction commit];
}

- (void)setPos:(CGPoint)p
{
	int pX = (int)p.x;
	int pY = (int)p.y;
	tX = (int)((pX - (pX % 23)) / 23);
	tY = (int)((pY - (pY % 23)) / 23);
	[mainLyr setFrame:CGRectMake(p.x,p.y,23.0,23.0)];
}

- (void)setPosNoAnim:(CGPoint)p
{
	[LKTransaction begin];
	[LKTransaction setValue:(id)kCFBooleanTrue forKey:@"disableActions"];
	[LKTransaction setValue:[NSNumber numberWithFloat:0.0] forKey:@"animationDuration"]; // for some reason just disableActions doesn't do it here. this seems more reliable anyway
	int pX = (int)p.x;
	int pY = (int)p.y;
	tX = (int)((pX - (pX % 23)) / 23);
	tY = (int)((pY - (pY % 23)) / 23);
	[mainLyr setFrame:CGRectMake(p.x,p.y,23.0,23.0)];
	[LKTransaction commit];
}

- (void)dealloc
{
	int i;
	for(i=0;i<4;i++)
	{
		[blocks[i] release];
		[glows[i] release];
	}
	[mainLyr release];
	[super dealloc];
}
@end