// sidebar view, displays score, upcoming pieces, etc., and provides "menu" button

#import "TrisSideView.h"

@interface TrisApp : UIApplication {}
- (void)openMenu;
- (UIAlphaAnimation *)buildFadeForTarget:(UIView *)targ fadeIn:(BOOL)i;
@end

@implementation TrisSideView

BOOL sPaused;

- (id)initWithFrame:(CGRect)frame app:(TrisApp *)a
{
	if(self = [super initWithFrame:frame])
	{
		app = a;
		
		float grf[4] = {1.0,1.0,1.0,0.7};
		float trf[4] = {0,0,0,0};
		float blf[4] = {0,0,0,0.4};
		
		CGColorSpaceRef csp = CGColorSpaceCreateDeviceRGB();
		CGColorRef c = CGColorCreate(csp,grf);
		CGColorRef t = CGColorCreate(csp,trf);
		CGColorRef b = CGColorCreate(csp,blf);
		
#ifdef ASPEN
		UIFont *lFont = [UIFont fontWithName:@"Helvetica" size:17];
#else
		struct __GSFont *lFont = [NSClassFromString(@"WebFontCache") createFontWithFamily:@"Helvetica" traits:2 size:17];
#endif
		
		bg = [[UIImage applicationImageNamed:@"side.png"] retain];
		
		menuImg = [[UIImage applicationImageNamed:@"menu.png"] retain];
		menuSelImg = [[UIImage applicationImageNamed:@"menu-sel.png"] retain];
		
		np = [[UIImage applicationImageNamed:@"next.png"] retain];
		
		// level
		lvlLbl = [[[UITextLabel alloc] initWithFrame:CGRectMake(11,106,68,24)] retain];
		[lvlLbl setFont:lFont];
#ifdef ASPEN
		[lvlLbl setColor:[UIColor colorWithCGColor:c]];
		[lvlLbl setBackgroundColor:[UIColor colorWithCGColor:t]];
		[lvlLbl setShadowColor:[UIColor colorWithCGColor:b]];
#else
		[lvlLbl setColor:c];
		[lvlLbl setBackgroundColor:t];
		[lvlLbl setShadowColor:b];
#endif
		[lvlLbl setCentersHorizontally:YES];
		[lvlLbl setText:@"1"];
		[lvlLbl setShadowOffset:CGSizeMake(0,1)];
		[lvlLbl setAlpha:0.0];
		
		// score
		scoreLbl = [[[UITextLabel alloc] initWithFrame:CGRectMake(11,166,68,24)] retain];
		[scoreLbl setFont:lFont];
#ifdef ASPEN
		[scoreLbl setColor:[UIColor colorWithCGColor:c]];
		[scoreLbl setBackgroundColor:[UIColor colorWithCGColor:t]];
		[scoreLbl setShadowColor:[UIColor colorWithCGColor:b]];
#else
		[scoreLbl setColor:c];
		[scoreLbl setBackgroundColor:t];
		[scoreLbl setShadowColor:b];
#endif
		[scoreLbl setCentersHorizontally:YES];
		[scoreLbl setText:@"0"];
		[scoreLbl setShadowOffset:CGSizeMake(0,1)];
		[scoreLbl setAlpha:0.0];
		
		[self addSubview:lvlLbl];
		[self addSubview:scoreLbl];
		
		// array to hold the next-piece blocks
		NSMutableArray *vus = [NSMutableArray arrayWithCapacity:4];

		int i;
		for(i=0;i<4;i++)
		{
			UIImageView *vu = [[[UIImageView alloc] initWithImage:np] retain];
			[vus addObject:vu];
			[vu setAlpha:0.0];
			[self addSubview:vu];
		}
		// ... and move its contents to an immutable
		nextPieceViews = [[NSArray arrayWithArray:vus] retain];
		
		[self setNextPiece:TRIS_L];
		
		sPaused = YES;

	}
	return self;
}

- (void)setScore:(int)sc
{
	[scoreLbl setText:[NSString stringWithFormat:@"%i",sc]];
}

- (void)setLevel:(int)lv
{
	[lvlLbl setText:[NSString stringWithFormat:@"%i",lv]];
}

- (void)setRows:(int)rw
{
	
}

- (void)moveView:(int)i toX:(int)x Y:(int)y
{
	[(UIView *)[nextPieceViews objectAtIndex:i] setFrame:CGRectMake((float)x,(float)y,13,13)];
}

- (void)setPaused:(BOOL)p
{
	sPaused = p;
	UIAnimator *ua = [UIAnimator sharedAnimator];
	NSMutableArray *fades = [NSMutableArray arrayWithCapacity:6];
	[fades addObject:[app buildFadeForTarget:scoreLbl fadeIn:!p]];
	[fades addObject:[app buildFadeForTarget:lvlLbl fadeIn:!p]];
	int i;
	for(i=0;i<4;i++)
	{
		[fades addObject:[app buildFadeForTarget:[nextPieceViews objectAtIndex:i] fadeIn:!p]];
	}
	[ua addAnimations:fades withDuration:0.3 start:YES];
	[self setNeedsDisplay];
}

#if defined ASPEN && !defined USE_LEGACY_EVENT

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{  
	UITouch *touch = [[event allTouches] anyObject];
	CGPoint p = [touch locationInView:self];
	if( p.y > 424 && p.y < 456 )
	{
		[app openMenu];
	}
	[self setNeedsDisplay];

}

#else
- (void)mouseUp:(GSEvent *)evt
{
	// I'm too lazy to get a UIPushButton to stay down... this is easier
	CGPoint p = GSEventGetLocationInWindow(evt);
	if(p.x > 230 && p.x < 320 && p.y > 424 && p.y < 456)
	{
		[app openMenu];
	}
	[self setNeedsDisplay];
}
#endif
- (void)setNextPiece:(int)k
{
	switch(k)
	{
		case TRIS_O:
			[self moveView:0 toX:30 Y:267];
			[self moveView:1 toX:45 Y:267];
			[self moveView:2 toX:30 Y:282];
			[self moveView:3 toX:45 Y:282];
			break;
		case TRIS_L:
			[self moveView:0 toX:30 Y:259];
			[self moveView:1 toX:30 Y:274];
			[self moveView:2 toX:30 Y:289];
			[self moveView:3 toX:45 Y:289];
			break;
		case TRIS_J:
			[self moveView:0 toX:45 Y:259];
			[self moveView:1 toX:45 Y:274];
			[self moveView:2 toX:45 Y:289];
			[self moveView:3 toX:30 Y:289];
			break;
		case TRIS_T:
			[self moveView:0 toX:31 Y:259];
			[self moveView:1 toX:31 Y:274];
			[self moveView:2 toX:31 Y:289];
			[self moveView:3 toX:46 Y:274];
			break;
		case TRIS_S:
			[self moveView:0 toX:31 Y:259];
			[self moveView:1 toX:31 Y:274];
			[self moveView:2 toX:46 Y:289];
			[self moveView:3 toX:46 Y:274];
			break;
		case TRIS_Z:
			[self moveView:0 toX:46 Y:259];
			[self moveView:1 toX:46 Y:274];
			[self moveView:2 toX:31 Y:289];
			[self moveView:3 toX:31 Y:274];
			break;
		case TRIS_I:
			[self moveView:0 toX:39 Y:252];
			[self moveView:1 toX:39 Y:267];
			[self moveView:2 toX:39 Y:282];
			[self moveView:3 toX:39 Y:297];
			break;
	}
}

- (void)setParent:(UIWindow *)parent
{
	prnt = parent;
}

- (void)drawRect:(CGRect)r
{
	[bg draw1PartImageInRect:CGRectMake(0,0,90,480)];
	if(!sPaused)
		[menuImg draw1PartImageInRect:CGRectMake(12,428,66,22)];
	else
		[menuSelImg draw1PartImageInRect:CGRectMake(12,428,66,22)];
	
}

- (void)dealloc
{
	int i;
	for(i=0;i<4;i++)
		[[nextPieceViews objectAtIndex:i] release];
	[nextPieceViews release];
	[menuImg release];
	[menuSelImg release];
	[bg release];
	[np release];
	[super dealloc];
}

@end